@@ -10,10 +10,11 @@ use num::{Bounded, Integer};
 
 use crate::zest_hash::zest4096;
 
-// The index to the current block will be left as a 896 bit index position,
-// The seed of the CSPRNG will be 3072 bits,
-// The magic bytes for the CSPRNG will be "Bytes are fun :)" in big endian,
-// 64 bit unsigned integers.
+// Notice: The seed of the PRNG also holds the counter, in the higher array indicies.
+// Keep your key/seed in the low bits, and the nonce after that if you wish.
+// The top 16 bytes for the CSPRNG will be "Bytes are fun :)" in big endian,
+// The next highest bytes will be counted over once the current 512 byte buffer
+// has been read from.
 
 const MAGIC_BYTES: [u64; 2] = [
     u64::from_be_bytes(*b"Bytes ar"),
@@ -31,6 +32,7 @@ pub struct ZestRand {
 }
 impl ZestRand {
     // (Re)seed the CSPRNG, and start at beginning of block + offset
+    #[inline(always)]
     pub fn seed(&mut self, seed: [u64; 62], byte_offset: usize) {
         if self.seed != seed {
             self.seed = seed;
@@ -38,8 +40,9 @@ impl ZestRand {
         }
         self.byte_offset = byte_offset & 0x1FF;
     }
-    // Counts to the next block, (lower index ptr = lower place value)
+    // Counts to the next block, (lower index ptr = higher place value)
     // and sets the hash output to None
+    #[inline(always)]
     fn next_block(&mut self) {
         let mut carry: u64 = 1;
         for elem in self.seed.iter_mut().rev() {
@@ -48,11 +51,13 @@ impl ZestRand {
         }
         self.hash_output = None;
     }
-    // Hash a new block and store it
-    fn compute_block(&mut self) {
-        self.hash_output = Some(zest4096(self.generate_input()));
+    // Hash the current block, return it for use.
+    #[inline(always)]
+    pub fn compute_block(&mut self) -> [u64; 64] {
+        zest4096(self.generate_input())
     }
     // Generate the input to the zest hash based on seed, index, and magic bytes
+    #[inline(always)]
     fn generate_input(&self) -> [u64; 64] {
         let mut block = [0_u64; 64];
         let (seed_split, magic_split) = block.split_at_mut(62);
@@ -67,64 +72,69 @@ impl ZestRand {
 // Functions which output random data:
 impl ZestRand {
     // Fetch the next byte, this constructs everything.
+    #[inline(always)]
     pub fn next_byte(&mut self) -> u8 {
         if self.hash_output.is_none() {
-            self.compute_block();
+            self.hash_output = Some(self.compute_block());
         }
-        if let Some(hash_out) = &mut self.hash_output {
-            // Safety: known size and type before and after conversion
-            let byte_hash_out: &mut [u8; 512] = unsafe { transmute(hash_out) };
-            let index = self.byte_offset & 0x1FF;
-            let byte_val = byte_hash_out[index];
-            self.byte_offset = index + 1;
-            if self.byte_offset >= 0x200 {
-                self.byte_offset &= 0x1FF;
-                self.next_block();
-            }
-            return byte_val;
+        // Safety: known size and type before and after conversion
+        let cast_slice: &[u8; 512] = unsafe { transmute(&self.hash_output) };
+        let byte_val = cast_slice[self.byte_offset & 0x1FF];
+        self.byte_offset += 1;
+        if self.byte_offset >= 0x200 {
+            self.byte_offset = 0;
+            self.next_block();
         }
-        0 // Default value if some serious error
+        byte_val
     }
     // Fills the output array with random bytes --
     // useful for types with "from_be_bytes" and "from_le_bytes" options.
+    #[inline(always)]
     pub fn next_bytes<const N: usize>(&mut self) -> [u8; N] {
         let mut output = [0; N];
         output.fill_with(|| self.next_byte());
         output
     }
-    ///# Safety
-    /// IMPORTANT! Only call on types without pointers, enumerations, and
-    /// types that would not be messed up with random initialization.
-    // Use to generate random integers.
-    pub fn next_num<T: Integer + Copy>(&mut self) -> T {
+    // Fill given type with random data.
+    /// # Safety
+    /// Do not use with objects containing pointers or references,
+    /// as they are written over with random data, and can cause a seg fault.
+    /// Do not use with enums, as not all values could map to a valid enum.
+    /// Think before you use!
+    #[inline(always)]
+    pub unsafe fn fill_rand<A: Copy>(&mut self) -> A {
         let mut data = vec![];
-        data.resize_with(size_of::<T>(), || self.next_byte());
-        let data_address = addr_of!(data[..]) as *const T;
-        unsafe { *data_address }
+        data.resize_with(size_of::<A>(), || self.next_byte());
+        let data_address = addr_of!(data[..]) as *const A;
+        *data_address
+    }
+    // Use to generate random integers.
+    #[inline(always)]
+    pub fn next_num<T: Integer + Copy>(&mut self) -> T {
+        unsafe { self.fill_rand() }
     }
     // Function takes in any signed integer type, and the max and minimum value it could be.
     // Returns an unbiased result in that type.
     // The maximum range is **not** checked to be the actual highest of the two arguments,
-    // the maximum range is also non-inclusive.
-    pub fn next_num_bounded<A: Integer + Copy + From<u8> + Bounded>(
-        &mut self,
-        minimum: A,
-        maximum: A,
-    ) -> A {
+    // the maximum range is also ***non-inclusive***.
+    #[inline(always)]
+    pub fn bounded<A: Integer + Copy + From<u8> + Bounded>(&mut self, min: A, max: A) -> A {
         let max_val: A = Bounded::max_value();
         let mut not_biased = self.next_num::<A>();
-        let range = maximum - minimum;
+        let range = max - min;
         while not_biased > (max_val - (((max_val % range) + A::from(1)) % range)) {
             not_biased = self.next_num::<A>();
         }
         if not_biased < A::from(0) {
-            // Calculate absolute value (works with signed and unsigned numbers)
+            // Calculate absolute value to make certain
+            // the numbers are not symetrical over the positive/negatice range
             not_biased = not_biased * (A::from(0) - A::from(1));
         }
-        not_biased % range + minimum
+        not_biased % range + min
     }
 }
 impl Default for ZestRand {
+    #[inline(always)]
     fn default() -> Self {
         // Safety: All fields are set to 0, and none
         // are pointers to any other object.
@@ -137,6 +147,7 @@ impl Default for ZestRand {
 // not assume that this function updates the original instance.
 impl Iterator for ZestRand {
     type Item = u8;
+    #[inline(always)]
     fn next(&mut self) -> Option<Self::Item> {
         Some(self.next_byte())
     }
@@ -145,11 +156,13 @@ impl Iterator for ZestRand {
 // Simply fills the requested buffer.
 // Always returns ok. You can safely unwrap.
 impl Read for ZestRand {
+    #[inline(always)]
     fn read(&mut self, buf: &mut [u8]) -> Result<usize> {
         buf.fill_with(|| self.next_byte());
         Ok(buf.len())
     }
     // The number of possible outputs is far greater than any ram stick array.
+    #[inline(always)]
     fn read_to_end(&mut self, _: &mut Vec<u8>) -> Result<usize> {
         panic!();
     }
@@ -157,9 +170,10 @@ impl Read for ZestRand {
 
 // Functions for shuffling slice types:
 impl ZestRand {
+    #[inline(always)]
     pub fn slice_shuffle<T: Sized + Copy>(&mut self, slice: &mut [T]) {
         for i in 0..slice.len() {
-            let swap_index = self.next_num_bounded(i, slice.len());
+            let swap_index = self.bounded(i, slice.len());
             slice.swap(i, swap_index);
         }
     }
